/* Copyright (c) 2002-2008, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT_MVAPICH in the top level MPICH directory.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "mv_dev.h"
#include "mv_param.h"
#include "mv_priv.h"
#include "mem_hooks.h"
#include "dreg.h"

void MPID_MV_Finalize(void) 
{
    int i;

#ifndef DISABLE_PTMALLOC
    mvapich_mfin();
#endif

    deallocate_mv_bufs();

    pmgr_finalize();

    while (dreg_evict());

    if(mvparams.use_zcopy) {
        /* destroy resources used for sending */
        for(i = 0; i < mvdev.num_hcas; i++) {
            ibv_destroy_qp(mvdev.rndv_qp[i].qp);
            ibv_destroy_cq(mvdev.rndv_cq[i]);
        }

        /* destroy resources used for recv */
        for(i = 0; i < mvparams.rndv_qps; i++) {
            ibv_destroy_qp(mvdev.rndv_pool_qps[i].ud_qp);
            ibv_destroy_cq(mvdev.rndv_pool_qps[i].ud_cq);
        }

        free(mvdev.rndv_pool_qps);
        free(mvdev.grh_buf);
        free(mvdev.rndv_cq);
        free(mvdev.rndv_qp);
    }

    for(i = 0; i < mvdev.np; i++) {
        int j;
        for(j = 0; j < mvparams.max_ah_total; j++) {
            ibv_destroy_ah(mvdev.connections[i].data_ud_ah[j]);
        }
        free(mvdev.connections[i].data_ud_ah);
    }

    for(i = 0; i < mvdev.num_ud_qps; i++) {
        ibv_destroy_qp(mvdev.ud_qp[i].qp);
    }
    free(mvdev.ud_qp);

    for(i = 0; i < mvdev.num_hcas; i++) {
        /* cancel the event thread */
        pthread_cancel(mvdev.hca[i].async_thread);
        pthread_join(mvdev.hca[i].async_thread, NULL);

        ibv_destroy_cq(mvdev.cq[i]);
        ibv_dealloc_pd(mvdev.hca[i].pd);
        ibv_close_device(mvdev.hca[i].context);
    }
    free(mvdev.cq);
    free(mvdev.hca);


    free(mvdev.lids);
    free(mvdev.qpns);
    free(mvdev.connections);
    free(mvdev.connection_ack_required);
    free(mvdev.pids);
    free(mvdev.my_name);

}



