/* Copyright (c) 2002-2008, The Ohio State University. All rights
 * reserved.
 *
 * This file is part of the MVAPICH software package developed by the
 * team members of The Ohio State University's Network-Based Computing
 * Laboratory (NBCL), headed by Professor Dhabaleswar K. (DK) Panda.
 *
 * For detailed copyright and licensing information, please refer to the
 * copyright file COPYRIGHT_MVAPICH in the top level MPICH directory.
 *
 */

#ifndef _MV_PARAM_H
#define _MV_PARAM_H

#include "ibverbs_header.h"

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

#define VIADEV_WTICK (0.000010)

typedef struct {
    uint8_t     use_headers;
    uint8_t     use_zcopy;
    uint8_t     use_lmc;
    int         max_lmc_total;
    uint8_t     max_rc_perconnection;
    uint16_t    send_window_segments_max;
    uint16_t    recv_window_size_max;
    uint16_t    rndv_qps;
    uint32_t    zcopy_threshold;
    uint32_t    rendezvous_threshold;
    int         send_ackafter;
    int         send_ackafter_progress;
    int         r3_segments_max;
    char device_name[32];

    long        retry_usec;
    long        progress_timeout;

    uint32_t    dreg_cache_limit;
    uint32_t    ndreg_entries;
    uint8_t     use_dreg_cache;

    uint32_t cq_size;
    uint32_t sq_size;
    uint32_t rq_size;
    uint32_t psn;
    int default_port;

    uint32_t max_ud_inline;

    int recvq_size;
    int credit_preserve;
    int credit_preserve_lazy;

    uint16_t max_retry_count;
    int num_qps;
    int mtu;

    int max_sl;
    int max_ah_total;
} mvdev_param_info;

#ifdef _SMP_
/* Run time parameter to disable/enable 
 *  * shared memory communicaiton */
extern int disable_shared_mem;
/* EAGER threshold in SMP path */
extern int                  smp_eagersize;
/* Shared memory buffer pool size */
extern int                  smpi_length_queue;
/*
 *  * Add affinity supprot. Disabled by default.
 *   */
#ifdef _AFFINITY_
extern unsigned int         viadev_enable_affinity;
extern char *cpu_mapping;
#endif

#endif

extern mvdev_param_info mvparams;


void MV_Init_Params(int num_proc, int me);

#endif
